package com.huiju.welcome.interceptor;

import com.alibaba.fastjson.JSONObject;
import org.apache.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.util.HashMap;
import java.util.Map;

/**
 * 登录校验拦截器
 * @ClassName: AccessInterceptor
 * @Description:
 * @author weiyb
 * @date 2018年5月7日 上午11:51:53
 */
@Component
public class AccessInterceptor implements HandlerInterceptor {
    // 正则白名单
    private String[] whiteList = {
            "/login",
            "/device",
            "/familiar",
            "/stranger",
            "/screen",
            "/camera",
            "/websocket"
    };

    /*
     * 视图渲染之后的操作
     */
    @Override
    public void afterCompletion(HttpServletRequest arg0, HttpServletResponse arg1, Object arg2, Exception arg3)
            throws Exception {

    }

    /*
     * 处理请求完成后视图渲染之前的处理操作
     */
    @Override
    public void postHandle(HttpServletRequest arg0, HttpServletResponse arg1, Object arg2, ModelAndView arg3)
            throws Exception {

    }

    /*
     * 进入controller层之前拦截请求
     */
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object obj) throws Exception {
        // 白名单跳过
        if (inWhiteList(request)) {
            return true;
        }

        HttpSession httpSession = request.getSession();
        String token = (String) httpSession.getAttribute("token");

        // 如果未登录
        if (null == token || "".equals(token)) {
            Map error = new HashMap<String, Object>();
            error.put("code", HttpStatus.SC_UNAUTHORIZED);
            error.put("message", "请先登录系统");
            response.addHeader("Content-type", "application/json");
            response.getOutputStream().write(JSONObject.toJSONBytes(error));
            return false;
        }

        return true;
    }

    private boolean inWhiteList(HttpServletRequest request) {
        String requestURI = request.getRequestURI();

        for (String it : whiteList) {
            if (requestURI.contains(it)) {
                return true;
            }
        }

        return false;
    }

}
