package com.huiju.welcome.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.mapper.TaDeviceMapper;
import com.huiju.welcome.mapper.TaPersonMapper;
import com.huiju.welcome.model.TaDevice;
import com.huiju.welcome.model.TaPerson;
import com.huiju.welcome.model.TaVisitingLog;
import com.huiju.welcome.service.ITaPersonService;
import com.huiju.welcome.service.ITaVisitingLogService;
import com.huiju.welcome.utils.StatusUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-03-12
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TaPersonServiceImpl extends ServiceImpl<TaPersonMapper, TaPerson> implements ITaPersonService {

    private Logger log = LoggerFactory.getLogger(TaPersonServiceImpl.class);

    @Autowired
    private TaPersonMapper taPersonMapper;

    @Autowired
    private ITaVisitingLogService taVisitingLogService;
    
    @Autowired
    private TaDeviceMapper taDeviceMapper;
    
    @Value("${huiju.files}")
    private String huijuFiles;

    @Override
    public String updateAvatar(TaPerson taPerson, TaDevice ...dev) {
        String state = "";

        List<TaDevice> taDeviceList = null;
        if (null == dev || dev.length == 0) {
            ByteArrayOutputStream bos = new ByteArrayOutputStream(1000);
            QueryWrapper<TaDevice> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("status", StatusUtils.Normal);
            taDeviceList = taDeviceMapper.selectList(queryWrapper);
        } else {
            taDeviceList = Arrays.asList(dev);
        }

        try {
            //分发人脸到各个相机
            RestTemplate restTemplate = new RestTemplate();
            //获取文件流
            String avatar = taPerson.getAvatar();
            if (avatar.startsWith("/")) {
                avatar = avatar.substring(1);
            }

            FileInputStream fileInputStream = new FileInputStream(new File(huijuFiles + avatar));
            byte[] byt = new byte[fileInputStream.available()];
            fileInputStream.read(byt);

            for(TaDevice taDevice : taDeviceList){
                String cameraUrl = taDevice.getApi() + "/video/" + taPerson.getPersonId();
                HttpHeaders headers = new HttpHeaders();
                headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
                HttpEntity httpEntity = new HttpEntity(byt);
                String result = restTemplate.exchange(cameraUrl, HttpMethod.POST, httpEntity, String.class).getBody();
                JSONObject jsonObject = JSONObject.parseObject(result);
                if (!jsonObject.get("code").equals(200)){
                    log.error(" 下发人脸失败: "+  jsonObject.get("message").toString());
                    return taDevice.getDeviceName() + "请检查状态！";
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "更新头像失败";
        }
        boolean status = taPersonMapper.updatePersonAvatar(taPerson);
        if (status){
            return "success";
        }
        return "更新头像失败";
    }

    @Override
    public Map<String, Object> detail(int id) {
        TaPerson taPerson = getById(id);

        QueryWrapper<TaVisitingLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("real_person", taPerson.getRealId());
        queryWrapper.orderByDesc("visite_date");

        List<TaVisitingLog> logs = taVisitingLogService.list(queryWrapper);

        Map<String, Object> res = new HashMap<>();
        res.put("detail", taPerson);
        res.put("visitingLog", logs);
        return res;
    }
    
    /**
     * 获取详细信息
     * @param personId
     * @param deviceId
     * @return
     */
    @Override
    public Map<String, String> getDetailInfo(Integer personId, String deviceId) {
        return taPersonMapper.getDetailInfo(personId,deviceId);
    }

    @Override
    public List<Map<String, String>> statsPerson() {
        return taPersonMapper.statsPerson();
    }

    @Override
    public boolean updateSales(TaPerson taPerson) {
        UpdateWrapper<TaPerson> wrapper = new UpdateWrapper<>();
        wrapper.eq("person_id", taPerson.getPersonId());
        wrapper.isNull("last_sales");

        return taPersonMapper.update(taPerson, wrapper) >= 0;
    }

    @Override
    public boolean mergePerson(Integer fromPerson, TaPerson toPerson) {
        // 合并基础信息
        UpdateWrapper<TaPerson> wrapper = new UpdateWrapper<>();
        wrapper.set("name", toPerson.getName());
        wrapper.set("sex", toPerson.getSex());
        wrapper.set("type_id", toPerson.getTypeId());
        wrapper.set("phone", toPerson.getPhone());
        wrapper.set("email", toPerson.getEmail());
        wrapper.set("remark", toPerson.getRemark());
        wrapper.set("real_id", toPerson.getRealId());
        wrapper.set("words", toPerson.getWords());
        wrapper.eq("person_id", fromPerson);

        boolean success = taPersonMapper.update(new TaPerson(), wrapper) >= 0;
        if (!success) return false;

        wrapper.eq("person_id", toPerson.getPersonId());
        success = taPersonMapper.update(new TaPerson(), wrapper) >= 0;
        if (!success) return false;

        // 合并访问记录
        TaVisitingLog taVisitingLog = new TaVisitingLog();
        taVisitingLog.setRealPerson(toPerson.getRealId());
        taVisitingLog.setPersonId(fromPerson);
        UpdateWrapper<TaVisitingLog> wrapper2 = new UpdateWrapper<>();
        wrapper2.eq("person_id", fromPerson);
        return taVisitingLogService.update(taVisitingLog, wrapper2);
    }

    @Override
    public IPage<TaPerson> getRealPersonList(Page<TaPerson> page, Integer personId, Integer typeId, String name, String phone, String beginDate, String endDate) {
        return taPersonMapper.getRealPersonList(page, personId, typeId, name, phone, beginDate, endDate);
    }
}
